@@ -96,3 +96,15 @@ class AfsNumberRefundEventConst(object):
             }
         }
     }
+
+
+class AfsLineGenerateFrom:
+    SELECT = 'select'  # 创建售后时选中的
+    VIRTUAL = 'virtual'  # 虚拟订单数据
+    ORIGIN = 'origin'  # 创建售后前的订单(或虚拟订单)数据
+
+    CHOICES = (
+        (SELECT, "创建售后时选择的商品"),
+        (VIRTUAL, "虚拟订单数据"),
+        (ORIGIN, "创建售后前的订单(或虚拟订单)数据"),
+    )
