@@ -4,6 +4,7 @@ from collections import defaultdict
 
 from django.db import models
 from django.db.models import Sum
+from django.core import validators
 from django.contrib.auth import get_user_model
 from django.utils.functional import cached_property
 
@@ -31,9 +32,34 @@ from ..sparrow_orders.models import LineOrderPromotion
 from ..sparrow_distribute.constants import DistributeStatus
 from ..sparrow_promotions.models import Coupon
 from .constants import Aftersale_Number_Refund_Status
+from .constants import AfsLineGenerateFrom
 from . import APP_LABEL
 
 
+__all__ = [
+    'Afs',
+    'AfsLine',
+    'AfsHgIncomeDetail',
+    'AfsPaymentBase',
+    'AfsPaymentCharge',
+    'AfsPaymentReturn',
+    'AfslinePaymentBase',
+    'AfslinePaymentCharge',
+    'AfslinePaymentReturn',
+    'AfsCouponBase',
+    'AfsCouponToCharge',
+    'AfsCouponToReturn',
+    'AfsNote',
+    'AfsAlipayRefund',
+    'DeliveryReturn',
+    'AfsHgIncome',
+    'AfsHgIncomeDivide',
+    'VLines',
+    'VLinePayment',
+    'VLinePromotion',
+]
+
+
 class Afs(BaseModelTimeAndDeleted):
     '''
     售后单
@@ -199,6 +225,9 @@ class Afs(BaseModelTimeAndDeleted):
     # 支付方法（once 一次支付/twice两次支付）
     pay_method = models.CharField("支付方法", max_length=24,choices=PayMethod.PAY_METHOD_CHOICES, blank=True, null=True, default=PayMethod.ONCE)
 
+    # 退货重算-售后单版本号
+    afs_version = models.CharField("售后版本号", max_length=12, blank=True, null=True, db_index=True)
+
     class Meta:
         app_label = APP_LABEL
         verbose_name = "Order After Sale"
@@ -539,14 +568,15 @@ class AfsLine(BaseModelTimeAndDeleted):
     line_id = models.PositiveIntegerField(
         'Line ID', blank=True, null=True, db_index=True)
     # quantity值只能是1，为了兼容老代码，保留此字段。
-    quantity = models.PositiveIntegerField('SKU退货数量', default=1)
+    # 虚拟订单上限, quantity 在虚拟单写入 afsline 时为负值
+    quantity = models.IntegerField('SKU退货数量', default=1)
     # 现金类退款金额, 包括支付宝，微信，银行卡，现金
     # 所有数量一共的退款
     # To be deleted by Jessica 2019-07-15 because of 售后单重构
     # cash_type_amount = models.DecimalField(
     #     "现金类金额", decimal_places=2, max_digits=12, default=0)
     # 系统要扣用户的积分（当年发给用户的，现在用户要补缴）
-    points_to_charge = models.PositiveIntegerField(
+    points_to_charge = models.IntegerField(
         '系统需要扣除用户的积分', default=0, blank=True, null=True)
     # 系统要还用户的C券（当年被用户用掉的，现在退货了要返还）--历史备注，暂时保留
     '''
@@ -556,7 +586,7 @@ class AfsLine(BaseModelTimeAndDeleted):
     因此在本次数据库调整中，points_to_return需要改回为“退还客户积分”的意义。
     改回前，需要把points_to_return“退还客户C券”的数据挪至sparrow_order_LineAfterSalePaymentReturn表中。
     '''
-    points_to_return = models.PositiveIntegerField(
+    points_to_return = models.IntegerField(
         '系统需要返还用户的积分', default=0, blank=True, null=True)
     # 系统要扣用户的A券（当年用户用掉的，现在用户要补缴）
     # To be deleted by Jessica 2019-07-15 because of 售后单重构
@@ -641,6 +671,12 @@ class AfsLine(BaseModelTimeAndDeleted):
     # 2021 年 9 月 对接微信退货组件新增字段 return_id
     return_id = models.CharField("微信组件退货ID", max_length=64, blank=True, null=True)
 
+    # 退货重算-售后版本号
+    afs_version = models.CharField("售后版本号", max_length=12, blank=True, null=True, db_index=True)
+    # 售后数据来源
+    generate_from = models.CharField(
+        "数据来源", max_length=12, blank=True, null=True, 
+        db_index=True, choices=AfsLineGenerateFrom.CHOICES, default=AfsLineGenerateFrom.SELECT)
 
     class Meta:
         # 同一个售后单里不应该创建两个相同发货单行的售后
@@ -1094,8 +1130,6 @@ class DeliveryReturn(BaseModelTimeAndDeleted):
         app_label = APP_LABEL
 
 
-
-
 class AfsHgIncome(BaseModelTimeAndDeleted):
     '''
     售后单营业外收入，与售后单Aftersale表关系为0/1对1，即一个售后单可以存在0个或者1个AfterSaleHgIncome
@@ -1135,4 +1169,204 @@ class AfsHgIncomeDivide(BaseModelTimeAndDeleted):
     #   支付宝支付：用户 支付宝支付ID
     payment_num = models.CharField('支付号', max_length=50, blank=True, null=True)
     class Meta:
-        app_label = 'sparrow_aftersale'
\ No newline at end of file
+        app_label = 'sparrow_aftersale'
+
+
+class VLines(BaseModelTimeAndDeleted):
+    aftersale_id = models.PositiveIntegerField('AfterSale ID', blank=True, null=True, db_index=True)
+    aftersale_number = models.CharField("退单号", max_length=40, db_index=True)
+    order_id = models.PositiveIntegerField('Order Id', blank=True, null=True, db_index=True)
+    order_number = models.CharField("订单编号", max_length=64, blank=True, null=True)
+    line_id = models.PositiveIntegerField('Line Id', blank=True, null=True, db_index=True)
+    # 品牌信息
+    brand_id = models.PositiveIntegerField('Brand Id', blank=True, null=True, db_index=True)
+    # 专柜信息
+    shop_id = models.PositiveIntegerField('Shop Id', blank=True, null=True, db_index=True)
+    # 柜号
+    shop_num = models.CharField("Shop number", max_length=128, blank=True)
+    # 厂商编码，货号
+    shop_sku = models.CharField("Shop SKU", max_length=128, blank=True, null=True)
+    # shop_name(专柜名称)
+    shop_name = models.CharField("Shop Name", max_length=36, blank=True, null=True)
+    # 汉光码 冗余进来
+    hg_code = models.CharField("汉光码", max_length=255, blank=True, null=True)
+    # 商品信息SKU
+    product_id = models.PositiveIntegerField(
+        'Product Id', blank=True, null=True, db_index=True)
+    # 商品名字
+    title = models.CharField("Product title", max_length=255)
+    # 订单里图片不单独上传，用商品数据库字段里的值
+    main_image = models.ImageField("Main Image", null=True, blank=True, max_length=255)
+    # sku属性——文字组合
+    sku_attr = models.CharField(
+        "SKU Attr", max_length=255, blank=True, null=True)
+    # 数量
+    quantity = models.PositiveIntegerField("Quantity", default=1)
+    # line里的总金额，不考虑活动，就是line本身的售价*数量
+    line_price = models.DecimalField("Price", decimal_places=2, max_digits=12)
+    # 活动优惠了的金额 —— 这个line全部数量单品里，满减和折扣这种能直接让钱变少的优惠金额
+    discount_amount = models.DecimalField(
+        "Discount", decimal_places=2, max_digits=12, blank=True, null=True, default=0)
+    # 汉光承担的优惠金额（比如原值活动的优惠）
+    discount_hg = models.DecimalField(
+        "汉光优惠金额", decimal_places=2, max_digits=12, blank=True, null=True, default=0)
+    # 专柜承担的优惠金额（比如专柜满减，折扣）
+    discount_shop = models.DecimalField(
+        "专柜优惠金额", decimal_places=2, max_digits=12, blank=True, null=True, default=0)
+    # 这个商品的原价（比如吊牌价）（1件的价格）
+    original_price = models.DecimalField(
+        "Original Price", decimal_places=2,
+        max_digits=12, blank=True, null=True)
+    # 这个商品的售价（1件的价格）
+    # 客户实际支付的金额为 line_price-discount_amount, 也即retail_price*quantity-discount_amount
+    retail_price = models.DecimalField(
+        "Retail Price", decimal_places=2, max_digits=12,
+        blank=True, null=True)
+    # 专柜收入 —— 这个line（此line总共）专柜的收入
+    shop_income_price = models.DecimalField(
+        "专柜收入", decimal_places=2, max_digits=12,
+        blank=True, null=True, default=0)
+    # 参与积分的金额 —— 可积分的金额（此line总共）
+    amount_for_point = models.DecimalField(
+        '用户实际支付金额——参与积分的金额',
+        default=Decimal('0'),
+        decimal_places=2,
+        max_digits=12,
+        validators=[validators.MinValueValidator(Decimal('0.00'))])
+    # 积分基数-金额 积分基数，比如1元20积分，这个字段就是1元。支持两位小数
+    point_amount = models.DecimalField(
+        '积分基数对应的金额，以元为单位',
+        default=Decimal('1'),
+        decimal_places=2,
+        max_digits=12,
+        validators=[validators.MinValueValidator(Decimal('0.01'))])
+    # 积分倍数 积分倍数，比如1元20积分，这个字段就是20。支持两位小数
+    # point_multiple = models.DecimalField(
+    #     '积分倍数',
+    #     default=Decimal('1'),
+    #     decimal_places=2,
+    #     max_digits=12,
+    #     validators=[validators.MinValueValidator(Decimal('0.01'))])
+    # 用户获取积分 此line总共用户应该获得的积分
+    points = models.PositiveIntegerField('Points', default=0)
+    # 中分类
+    offer = models.CharField("中分类", max_length=2, default="0")
+    # 商品条码
+    barcode = models.CharField("Product Barcode", max_length=256, blank=True, null=True)
+    # 商品信息SPU
+    productmain_id = models.PositiveIntegerField('Product Main Id', blank=True, null=True, db_index=True)
+    # giftproduct_id(赠品id)
+    giftproduct_id = models.PositiveIntegerField('赠品 ID', blank=True, null=True, db_index=True)
+    is_gift = models.BooleanField("是否是赠品", default=False, db_index=True)
+
+    class Meta:
+        app_label = 'sparrow_aftersale'
+
+
+class VLinePayment(models.Model):
+    ''' 虚拟订单的支付方式 '''
+
+    CouponSourceChoices = (
+        (CouponSource.ONLINE, '线上'),
+        (CouponSource.IN_STORE, '店内')
+    )
+
+    id = models.CharField("id", primary_key=True, max_length=40, default=get_uuid4_hex)
+    # 虚拟订单行的 ID
+    vline_id = models.PositiveIntegerField('VLine ID', blank=True, null=True, db_index=True)
+    aftersale_id = models.PositiveIntegerField('AfterSale ID', blank=True, null=True, db_index=True)
+    aftersale_number = models.CharField("退单号", max_length=40, db_index=True)
+    order_id = models.PositiveIntegerField('Order Id', blank=True, null=True, db_index=True)
+    order_number = models.CharField("订单编号", max_length=64, blank=True, null=True)
+    line_id = models.PositiveIntegerField('Line Id', blank=True, null=True, db_index=True)
+    # order_number_pay 子支付订单编号(订单编号加“-1”或者“-2”)
+    order_number_pay = models.CharField("子支付订单编号", blank=True, null=True, max_length=64)
+    # pay_step_type 支付阶段类型(all全款/deposit定金/tailpay尾款)
+    pay_step_type = models.CharField("Pay Step Type", max_length=24, choices=PayStepType.PAY_STEP_TYPE_CHOICES, default=PayStepType.ONCE, db_index=True)
+    # coupon_name 券名
+    coupon_name = models.CharField("券名", max_length=128, blank=True, null=True)
+    # 支付类型
+    payment_type = models.CharField("Payment type", max_length=10, blank=True, null=True)
+    # 如果是钱类的，就存金额。如果是积分支付，就用amount存积分
+    amount = models.DecimalField("Amount", decimal_places=2, max_digits=12)
+    # 用来记录支付的详细信息：
+    #   优惠券：优惠券ID coupon_id
+    #   店内优惠券存 code
+    #   C券：用户 account_id
+    #   微信支付：用户 微信支付ID
+    #   支付宝支付：用户 支付宝支付ID
+    payment_num = models.CharField('支付号', max_length=50, blank=True, null=True)
+    # 如果是券的话，券的来源
+    coupon_source = models.CharField(
+        '来源', max_length=64, blank=True, null=True,
+        choices=CouponSourceChoices, default=CouponSource.IN_STORE)
+    note = models.CharField('备注', max_length=200, blank=True, null=True)
+    # 如果支付方式是礼券，对应于：S001_OrderPayDetail中的Objid，其他默认为0
+    # 信息部的字段
+    list_id = models.IntegerField('List ID', default=0)
+
+    class Meta:
+        app_label = 'sparrow_aftersale'
+
+
+class VLinePromotion(models.Model):
+    ''' 虚拟订单的活动 '''
+
+    id = models.CharField("id", primary_key=True, max_length=40, default=get_uuid4_hex)
+    # 虚拟订单行的 ID
+    vline_id = models.PositiveIntegerField('VLine ID', blank=True, null=True, db_index=True)
+    aftersale_id = models.PositiveIntegerField('AfterSale ID', blank=True, null=True, db_index=True)
+    aftersale_number = models.CharField("退单号", max_length=40, db_index=True)
+    order_id = models.PositiveIntegerField('Order Id', blank=True, null=True, db_index=True)
+    order_number = models.CharField("订单编号", max_length=64, blank=True, null=True)
+    line_id = models.PositiveIntegerField('Line Id', blank=True, null=True, db_index=True)
+    # 活动ID
+    promotion_id = models.PositiveIntegerField(
+        'Promotion Id', blank=True, null=True, db_index=True)
+
+    # 主活动ID
+    promotionmain_id = models.PositiveIntegerField(
+        'Promotion Main Id', blank=True, null=True, db_index=True)
+    # 活动类型
+    promotion_type = models.CharField(
+        "活动类型", max_length=30, blank=True, null=True)
+
+    # 活动标签
+    promotion_label = models.CharField(
+        "活动标签", max_length=20, blank=True, null=True)
+    # 活动序号
+    promotion_seq = models.CharField(
+        "活动序号", max_length=2, blank=True, null=True)
+    # 活动名称
+    promotion_name = models.CharField(
+        "活动名称", max_length=20, blank=True, null=True)
+    # 活动摘要，把对应档的活动描述复制到这里 —— 有可能在订单详情展示
+    summary = models.CharField("Promotion Summary",
+                               max_length=256, blank=True)
+
+    class Meta:
+        app_label = 'sparrow_aftersale'
+
+
+class VlinesOnlyRefund(models.Model):
+    ''' 虚拟订单行对应的该行商品的总仅退差价额 '''
+    CouponSourceChoices = (
+        (CouponSource.ONLINE, '线上'),
+        (CouponSource.IN_STORE, '店内')
+    )
+    id = models.CharField("id", primary_key=True, max_length=40, default=get_uuid4_hex)
+    vline_id = models.PositiveIntegerField('VLine ID', blank=True, null=True, db_index=True)
+    aftersale_id = models.PositiveIntegerField('AfterSale ID', blank=True, null=True, db_index=True)
+    aftersale_number = models.CharField("退单号", max_length=40, db_index=True)
+    order_id = models.PositiveIntegerField('Order Id', blank=True, null=True, db_index=True)
+    order_number = models.CharField("订单编号", max_length=64, blank=True, null=True)
+    line_id = models.PositiveIntegerField('Line Id', blank=True, null=True, db_index=True)
+    payment_type = models.CharField("Payment type", max_length=10, blank=True, null=True)
+    amount = models.DecimalField("Amount", decimal_places=2, max_digits=12)
+    coupon_id = models.CharField('Coupon ID', max_length=50, blank=True, null=True, db_index=True)
+    coupon_number = models.CharField('Coupon Number', max_length=64, blank=True, null=True, db_index=True)
+    coupon_source = models.CharField('来源', max_length=64, default=CouponSource.IN_STORE, choices=CouponSourceChoices)
+    pay_step_type = models.CharField("Pay Step Type", max_length=24, choices=PayStepType.PAY_STEP_TYPE_CHOICES, default=PayStepType.ONCE, db_index=True)
+
+    class Meta:
+        app_label = 'sparrow_aftersale'
